Namespace("Data");

Data.DataCacheManager = (function() {
	
	const DATA_CACHE_EXPIRY_MINUTES = 30; // 30 minutes
	const DATA_CACHE_NUM_LIMIT = 60; // max number of items to cache, if limit is hit - the stack will be reduced to 25 items
	
	var DataCache = new Array();
	
	/* START PRIVATE PROPERTIES */
	
	CreateDataCacheObject = function(key, cacheData) {
		
		var expiry = new Date(); expiry.setMinutes(expiry.getMinutes() + DATA_CACHE_EXPIRY_MINUTES);
		
		return {
			"key" : key,
			"data" : cacheData,
			"expiry" : expiry
		}
	}
	
	GetDataCacheSize = function() {
	    var size = 0, key;
	    for (key in DataCache) {
	        if (DataCache.hasOwnProperty(key)) size++;
	    }
	    return size;
	};
	
	ShrinkDataCache = function() {
		
		var newCacheData = [];
		
		for (key in DataCache) {
	        if(typeof(DataCache[key]) != "undefined") 
			{
				newCacheData.push(DataCache[key]);
			}
	    }
		
		// sort the array with descending expiry dates
		newCacheData.sort(function(a,b)
		{
			return a.expiry < b.expiry
		});
		
		// remove the items expiring the soonest, and leave 25 items
		var newLength = 35;
		if(newCacheData.length > newLength) {
			newCacheData = newCacheData.slice(0,newLength-1);
			DataCache = new Array();
			for (var i = 0; i < newCacheData.length; i++) {
				DataCache[newCacheData[i].key] = newCacheData[i];
			}
		}
	}
	
	SaveCacheData = function(key, cacheData) {
		var cacheObj = CreateDataCacheObject(key,cacheData);
		DataCache[key] = cacheObj;
		
		if(GetDataCacheSize()> DATA_CACHE_NUM_LIMIT) 
		{
			// reached item limit, remove the oldest cache item
			ShrinkDataCache();
		}
	};
	
	GetUnexpiredCacheData = function(key) {
		var result = DataCache[key];
		
		if(result != null && result.expiry != null && result.expiry > (new Date())) {
			return result.data;
		}
		else {
			return null;
		}
	};
	
	// TODO: clear out expired data
	
	/* END PRIVATE PROPERTIES */

	/* START PUBLIC METHODS */
	
	/*
	GetPopularPhotos = function(callback, fullRefresh) {
		
		fullRefresh = typeof fullRefresh !== 'undefined' ? fullRefresh : false; // default to false
		
		var result = GetUnexpiredCacheData("POPULAR");
		
		if(result == null || fullRefresh) {
			Data.DataServiceMain.GetPopularPhotos(function(data){
				var modelData = Data.ModelFactory.CreatePhotosList(data.photos);
				SaveCacheData("POPULAR", modelData);
				callback(modelData);
			});
		}
		else {
			callback(result);
		}
		
	};
	
	GetPhotoDetailsByID = function(photo_id, callback, fullRefresh) {
		
		fullRefresh = typeof fullRefresh !== 'undefined' ? fullRefresh : false; // default to false
		
		var result = GetUnexpiredCacheData("PHOTO_DETAILS:" + photo_id);
		
		if(result == null || fullRefresh) {
			Data.DataServiceMain.GetPhotoDetailsByID(photo_id,function(data){
				var modelData = Data.ModelFactory.CreatePhotoModel(data.photo);
				SaveCacheData("PHOTO_DETAILS:" + photo_id, modelData);
				callback(modelData);
			});
		}
		else {
			callback(result);
		}
	};
	*/
	
	/* END METHODS */
	
	/* RETURN INSTANCE OBJECT -- WITH PUBLIC MEMBERS */
	return {
		/*
		GetPopularPhotos : GetPopularPhotos,
		GetPhotoDetailsByID : GetPhotoDetailsByID
		*/
	};

})();