Namespace("Data");

Data.DataServiceMain = (function() {
	
	/* START METHODS */
    const TUNEIN_BASE_URL = "http://opml.radiotime.com/";
    const ACCOUNT_SERVICE_URL = "https://opml.radiotime.com/" + "Account.ashx";
    const BROWSE_SERVICE_URL = TUNEIN_BASE_URL + "Browse.ashx";
    const TUNE_SERVICE_URL = TUNEIN_BASE_URL + "Tune.ashx";
    const OPTIONS_SERVICE_URL = TUNEIN_BASE_URL + "Options.ashx";
    const DESCRIBE_SERVICE_URL = TUNEIN_BASE_URL + "Describe.ashx";
    const SEARCH_SERVICE_URL = TUNEIN_BASE_URL + "Search.ashx";
    const PRESET_SERVICE_URL = "https://opml.radiotime.com/" + "Preset.ashx";
	
	CreateServiceParamsObj = function(options) {
		
		var basicParams = {
			"render" : "json",
			"partnerId" : Utils.Constants.PARTNER_ID,
			"serial" : SettingsManager.Instance.GetDeviceID(),
			"formats" : "mp3,aac",
			"timestamp" : Date.now()
		}
		
		return $.extend(basicParams,options);
	}
	
	IsDataValid = function(data) {
		return (data != null && 
				data.head != null && 
				data.head.status != null &&
				data.head.status == "200" &&
				data.body != null)
				
	}
	
	DoServiceCall = function(url, params, callback, resultFormat) {

		if(typeof(resultFormat) == "undefined")
			resultFormat = "json";
		
		$.get(url, params, function(data) {
        	
        	if(IsDataValid(data))
        	{
        		callback(data);
    		}
        	else
        	{
        		callback(null);
        	}
        }, resultFormat);
	}
	
	DoServiceCallPost = function(url, params, callback, resultFormat) {
		
		if(typeof(resultFormat) == "undefined")
			resultFormat = "json";

		$.post(url, params, function(data) {
        	
        	if(IsDataValid(data))
        	{
        		callback(data);
    		}
        	else
        	{
        		callback(null);
        	}
        }, resultFormat);
	}
	
	DoServiceCallText = function(url, params, callback) {
		
		$.get(url, params, function(data) {
        	callback(data);
        }, "text");
        
	}
	
	GetBrowseCategories = function(url, callback) { 
            var params = { "serial" : SettingsManager.Instance.GetDeviceID()};
           $.get(url, params,"xml")
                .success(function(data) { 
                    callback(data);
                 })
                .error(function(jqXHR, textStatus, errorThrown) {
                    if (jqXHR.status === 0) {
                       bb.pushScreen('views/error-connection.html', 'error-connection');
                    }
           });
	}
	
	// GetBrowseCategories = function(categoryId, callback) {
// 		
		// var url = BROWSE_SERVICE_URL;
		// var params = CreateServiceParamsObj({
			// c : categoryId
		// });
// 		
        // DoServiceCall(url, params, callback);
	// }
	
	GetNowPlayingInfo = function(stationId, callback) {
		
		var url = DESCRIBE_SERVICE_URL;
		var params = CreateServiceParamsObj({
			"c" : "nowplaying",
			"id" : stationId,
			"detail" : "genre"
		});
		
        DoServiceCall(url, params, callback)
	}
	
	GetNowPlayingInfoFull = function(stationId, callback) {
		
		var url = DESCRIBE_SERVICE_URL;
		var params = CreateServiceParamsObj({
			"id" : stationId,
		});
		
        DoServiceCall(url, params, callback)
	}
	
	GetStationTuneUrl = function(stationId, callback) {
		
		var url = TUNE_SERVICE_URL;
		var params = CreateServiceParamsObj({
			"id" : stationId,
			"render" : "",
			"formats" : ""
		});
		
		DoServiceCallText(url, params, callback);
		
	}
	
	GetStationTuneFromUrl = function(url, callback) {
		DoServiceCallText(url, null, callback);
	}
	
	GetStationStreams = function(stationId, callback) {
		var url = TUNE_SERVICE_URL;
		
		var params = CreateServiceParamsObj({
			"c" : "ebrowse",
			"id" : stationId
		});
		
        DoServiceCall(url, params, callback);
	}
	
	GetStationOptions = function(stationId, callback) {
		
		var url = OPTIONS_SERVICE_URL;
		var params = CreateServiceParamsObj({
			"c" : "",
			"id" : stationId
		});
		
        DoServiceCall(url, params, callback);
	}
	
	GetSearchResults = function(query, callback) {
        
		var url = SEARCH_SERVICE_URL;
		var params = CreateServiceParamsObj({
			"query" : query,
            "formats" : "mp3,aac"
		});
		
        DoServiceCall(url, params, callback);
        
	}

	JoinUserToDevice = function(userID, userPW, callback)
	{
		if(!Utils.GeneralUtils.IsNullOrEmpty(userID) && !Utils.GeneralUtils.IsNullOrEmpty(userPW))
		{
			var url = ACCOUNT_SERVICE_URL;

			var params = CreateServiceParamsObj({
				"c" : "join",
				"username" : userID,
				"password" : userPW,
				"serial" : SettingsManager.Instance.GetDeviceID()
			});

			DoServiceCallPost(url, params, callback);
		}
	}

	GetStreamHeaders = function(streamUrl, callback)
	{
		var requestObj = $.get(streamUrl, {"Icy-MetaData" : 1}, function(data) {
    		console.log(requestObj.getAllResponseHeaders());
    		callback(data, requestObj);
    		requestObj.abort();
        });
	}
	
	GetFavorites = function(callback) {
		
		var url = BROWSE_SERVICE_URL;
		var params = CreateServiceParamsObj({
			"c" : "presets"
		});
		
        DoServiceCall(url, params, callback);
	}
	
	UpdatePreset = function(stationId,mode,callback) {
		var url = PRESET_SERVICE_URL;
		var params = CreateServiceParamsObj({
			"c" : mode,
			"id" : stationId,
			"username" : ""
			// , "password" : ""
		});

		if(!Utils.GeneralUtils.IsNullOrEmpty(SettingsManager.Instance.GetUserID()))
		{
			params.username = SettingsManager.Instance.GetUserID();
		}
		
        DoServiceCall(url, params, callback);
	}
	
	DoRequestToUrl = function(url, callback) {
		var params ={
			"render" : "json"
		};
		
        DoServiceCall(url, params, callback);
	}
	
	/* RETURN INSTANCE OBJECT -- WITH PUBLIC MEMBERS */
	return {
		GetBrowseCategories : GetBrowseCategories,
		GetNowPlayingInfo : GetNowPlayingInfo,
		GetStationTuneUrl : GetStationTuneUrl,
		GetStationTuneFromUrl : GetStationTuneFromUrl,
		GetStationStreams : GetStationStreams,
		GetStationOptions : GetStationOptions,
		GetFavorites : GetFavorites,
		GetNowPlayingInfoFull : GetNowPlayingInfoFull,
		GetSearchResults : GetSearchResults,
		JoinUserToDevice : JoinUserToDevice,
		GetStreamHeaders : GetStreamHeaders,
		UpdatePreset : UpdatePreset,
		DoRequestToUrl : DoRequestToUrl
	};

})();