Namespace("SettingsManager");

SettingsManager.Languages = {
	English : "en",
	French : "fr",
	Spanish : "es"
}
	
SettingsManager.Instance = (function() {

	/* Constants */
	const KEY_APPLICATION_ID = "KEY_APPLICATION_ID";
	const KEY_DEVICE_ID = "KEY_DEVICE_ID";
	const KEY_USER_ID = "KEY_USER_ID";
	const KEY_USER_PW = "KEY_USER_PW";
	const KEY_LANGUAGE = "KEY_LANGUAGE";
    const KEY_LOGOS = "KEY_LOGOS";
        
	
	/* Settings Properties */
	var ApplicationID = null;
	var DeviceID = null;
	var UserID = null;
	var Language = null;
	
	/* START METHODS */
	
	LoadSettings = function() {
		ApplicationID = Manager.LocalStorageManager.getItem(KEY_APPLICATION_ID);
		DeviceID = Manager.LocalStorageManager.getItem(KEY_DEVICE_ID);
		UserID = Manager.LocalStorageManager.getItem(KEY_USER_ID);
		Language = Manager.LocalStorageManager.getItem(KEY_LANGUAGE);
        Logos = Manager.LocalStorageManager.getItem(KEY_LOGOS);
		
		if(ApplicationID == null){
			ApplicationID = Utils.GeneralUtils.GenerateGuid();
			Manager.LocalStorageManager.setItem(KEY_APPLICATION_ID, ApplicationID);
		}

		if(DeviceID == null){
			ResetDeviceID();
		}

		if(UserID == null){
			UserID = "";
			Manager.LocalStorageManager.setItem(KEY_USER_ID, UserID);
		}

		if(Language == null){
			Language = SettingsManager.Languages.English;
			Manager.LocalStorageManager.setItem(KEY_LANGUAGE, Language);
		}
                
        if(Logos == null){
			Logos = true;
			Manager.LocalStorageManager.setItem(KEY_LOGOS, Logos);
		}
	}
	
	SaveSettings = function() {
		Manager.LocalStorageManager.setItem(KEY_APPLICATION_ID, ApplicationID);
		Manager.LocalStorageManager.setItem(KEY_DEVICE_ID, DeviceID);
		Manager.LocalStorageManager.setItem(KEY_USER_ID, UserID);
		Manager.LocalStorageManager.setItem(KEY_LANGUAGE, Language);
        Manager.LocalStorageManager.setItem(KEY_LOGOS, Logos);
	}
	
	/* END METHODS */
	
	/* ACESSORS */
	
	GetApplicationID = function() {
		return ApplicationID;
	}
	
	GetDeviceID = function() {
		return DeviceID;
	}

	ResetDeviceID = function() {
		DeviceID = Utils.GeneralUtils.GenerateUserID();
		Manager.LocalStorageManager.setItem(KEY_DEVICE_ID, DeviceID);
	}

	GetUserID = function() {
		return UserID;
	}
	
	GetLanguage = function() {
		return Language;
	}
        
    GetLogosPref = function() {
		return Logos;
	}
	
	/* END ACCESSORS */
	
	/* START SETTERS */
	
	SetUserID = function(value) {
		UserID = value;
		SaveSettings();
	}

	SetLanguage = function(value) {
		Language = value;
		SaveSettings();
	}
        
    SetLogosPref = function(value) {
		Logos = value;
		SaveSettings();
	}
	
	/* END SETTERS */
	
	/* RETURN INSTANCE OBJECT -- WITH PUBLIC METHODS */
	return {
		LoadSettings : LoadSettings,
		SaveSettings : SaveSettings,
		GetApplicationID : GetApplicationID,
		GetDeviceID: GetDeviceID,
		ResetDeviceID : ResetDeviceID,
		GetUserID: GetUserID,
		SetUserID: SetUserID,
		GetLanguage : GetLanguage,
		SetLanguage : SetLanguage,
        GetLogosPref : GetLogosPref,
        SetLogosPref : SetLogosPref
	};

})();