/*
 * This script is for the favorites screen
 */

Namespace("Views");

Views.FavoritesPage = (function() {

    var PageRoot;
    var favoritesList;

    function Init(url) {

        PageRoot = document.getElementById("screen-favorites"); // Root element of this screen

        Views.FavoritesPage.doLoadFavorites(url);

        UpdateMiniPlayerContent();
        PlayerModule.addListener("playerStateUpdated", UpdateMiniPlayerContent);
    }

    function UpdateMiniPlayerContent()
    {
        $(PageRoot).find("#tunein-playerbar").html("");
        $(PageRoot).find("#tunein-playerbar").append(PlayerModule.GetMiniPlayerDiv());
    }

    function doLoadFavorites(url) {

        //clear any feedback we might have
        ActivityManager.ShowFeedback("");

        var userId = SettingsManager.Instance.GetUserID();

        favoritesList = new Array();

        favoritesList = JSON.parse(Manager.LocalStorageManager.getItem("FAVORITES_LIST"));

        //if it's null, we don't have favorites yet, so let's create a blank 
        //one to initalize our store
        if (favoritesList == null) {
            var temp = new Array();
            var i = 0;
            temp[i] = "";
            Manager.LocalStorageManager.setItem("FAVORITES_LIST", JSON.stringify(temp));
            favoritesList = JSON.parse(Manager.LocalStorageManager.getItem("FAVORITES_LIST"));
        }

        ActivityManager.ShowFeedback("Loading...");
        if(url != null){
            Data.DataServiceMain.DoRequestToUrl(url, buildFavorites);
        } else {
            Data.DataServiceMain.GetFavorites(buildFavorites);
        }
    }
    
    function buildFavorites(responseData) {
            var text, audioType, guide_id, playing_image, bitrate = "";
            if (responseData != null) {
                ActivityManager.HideIndicator();
                favoritesData = responseData.body;

                //clear our list of any old results
                var dataList = document.getElementById('favoritesDataList');
                dataList.innerHTML = "";
                // Create our list item
                for (var idx in responseData.body) {
                    var category = responseData.body[idx];
                    var listItem = CreateListItem(category);
                    dataList.appendItem(listItem);
                    if (category.children != null) {
                        for (var i = 0; i < category.children.length; i++){
                            var item = category.children[i];
                            var listItem = CreateListItem(item);
                            dataList.appendItem(listItem);
                        }
                    }
                }

                //save the favorites list to localstore
                Manager.LocalStorageManager.setItem("FAVORITES_LIST", JSON.stringify(favoritesList));

                ActivityManager.ShowFeedback("");
            } else {
                ActivityManager.ShowFeedback("_Service unavailable. Please try again later.");
            }
        }
        
        
    function CreateListItem(item) {
        var listItem = document.createElement('div');

        var itemType = $(item).attr('type');

        if (itemType != null && itemType == "link")
        {
            listItem.setAttribute('data-bb-type', 'item');
            listItem.setAttribute('data-bb-title', $(item).attr('text'));
            listItem.setAttribute('onclick', "bb.pushScreen('views/favoritesSubcategory.html', 'favoritesSubcategory', {url:'" + $(item).attr('URL') + "'});");
            listItem.setAttribute("data-tunein-type", "no_image_item");
        }
        else if (itemType != null && itemType == "audio")
        {
            playing_image = $(item).attr('image');
            bitrate = $(item).attr('bitrate');
            guide_id = $(item).attr('guide_id');

            listItem.setAttribute('data-bb-type', 'item');
            listItem.setAttribute('data-bb-img', $(item).attr('image'));
            listItem.setAttribute('data-bb-title', $(item).attr('text'));
            
            if($(item).attr("text").toLowerCase().indexOf("not supported") == -1)
            {
	            listItem.setAttribute('onclick', "bb.pushScreen('views/player.html', 'playerView',{stationId : '" + guide_id + "',playing_image : '" + playing_image + "' });");
	        }
	        
            listItem.innerHTML = $(item).attr('subtext');

        }
        else {
            if(!Utils.GeneralUtils.IsNullOrEmpty($(item).attr('text')))
            {
                listItem.setAttribute('class', 'sectionheader');
                listItem.innerHTML = $(item).attr('text');    
            }
        }

        return listItem;
    }    
    
    function refresh() {
        document.getElementById('favoritesDataList').innerHTML = "";
        ActivityManager.ShowIndicator();
        Init();
    }

    return {
        Init: Init,
        doLoadFavorites: doLoadFavorites,
        refresh: refresh,
        CreateListItem: CreateListItem 
    }
})();

//** sample response */
/*
 
 <?xml version="1.0" encoding="UTF-8"?>
 <opml version="1">
 <head>
 <title>My Favorites</title>
 <status>200</status>
 
 </head>
 <body>
 <outline type="audio" text="A2R - Rap US (Hip Hop)" URL="http://opml.radiotime.com/Tune.ashx?id=s150106&amp;partnerId=GdTO4j7I&amp;serial=u50123819" bitrate="128" reliability="94" guide_id="s150106" subtext="Chamillionaire - Turn It Up (feat. Lil&apos; Flip)" genre_id="g128" formats="mp3" playing="Chamillionaire - Turn It Up (feat. Lil&apos; Flip)" playing_image="http://d3tybumvrk5xfv.cloudfront.net/409734088d.jpg" item="station" image="http://d1i6vahw24eb07.cloudfront.net/s150106q.png" now_playing_id="s150106" preset_number="1" preset_id="s150106" is_preset="true"/>
 <outline type="audio" text="West Coast 4 You (Hip Hop)" URL="http://opml.radiotime.com/Tune.ashx?id=s145209&amp;partnerId=GdTO4j7I&amp;serial=u50123819" bitrate="128" reliability="95" guide_id="s145209" subtext="Daz Dillinger - My Mama Said..." genre_id="g128" formats="mp3" playing="Daz Dillinger - My Mama Said..." playing_image="http://d3tybumvrk5xfv.cloudfront.net/860582042d.jpg" item="station" image="http://d1i6vahw24eb07.cloudfront.net/s145209q.png" now_playing_id="s145209" preset_number="2" preset_id="s145209" is_preset="true"/>
 </body>
 </opml>
 */

