Namespace("PlayerModule");

PlayerModule = (function() {

    var PlayerStates = {
        STOPPED: "STOPPED",
        PLAYING: "PLAYING",
        LOADING: "LOADING",
        ERROR: "ERROR"
    }

    var player = null;

    // private vars
    var currentStation = null; // station metadata
    var streamOptions = null;
    var currentStream = null;
    var playerState;
    var currentBitrate = "";
    var currentFormat = "";
    var secondsRemaining = 0;
    var playDuration = 0;
    var playDurationDelta = 0;
    var duration = 0;
    var twitterHandle = "";
    var stationLogo = "";
    var miniPlayerLogo = "";
    var timeUpdateInterval = -1;
    var isPreset = false;

    var html5Audio = null;

    var ready = false;

    // Now Playing Info

    var stationName = "";
    var showName = "";
    var songName = "";
    var albumName = "";
    var artistName = "";
    var genre = "";
    var country = "";

    var nowPlayingInterval = -1;
    var stalledStatonCancelTimer = -1;

    var _events = {};


    addListener = function(eventName, callback) {
        var events = _events,
                callbacks = events[eventName] = events[eventName] || [];

        if (callbacks.indexOf(callback) == -1)
        {
            callbacks.push(callback);
        }
    }

    raiseEvent = function(eventName, args) {
        var events = _events;
        var callbacks = events[eventName] = events[eventName] || [];
        for (var i = 0, l = callbacks.length; i < l; i++) {
            callbacks[i](eventName, args);
        }
    }

    OnPlayerPause = function()
    {
        $("#jquery_jplayer_1").jPlayer("clearMedia");

        if (window["CURRENT_PLAYER_STATE"] != PlayerStates.ERROR)
        {
            console.log("STOPPED");
            window["CURRENT_PLAYER_STATE"] = PlayerStates.STOPPED;
            DispatchPlayerStateUpdatedEvent();
        }
    }

    OnPlayerPlay = function()
    {
        console.log("PLAYING");

        if (stalledStatonCancelTimer != -1)
        {
            clearTimeout(stalledStatonCancelTimer);
        }
    }

    SetupTimeUpdateInterval = function()
    {
        timeUpdateInterval = setInterval(function() {
            playDurationDelta += 1;
            var playPercent = 0;
            // if data available from the feed on time remaining
            if (duration > 0) {
                playDuration = duration - secondsRemaining + playDurationDelta;
                playPercent = Math.round(playDuration * 100 / duration);
            }
            else {
                playDuration = playDurationDelta;
            }

            var playDurationFormatted = Utils.GeneralUtils.FormatPlayDuration(playDuration);

            $("#progressinfo #timeelapsed").html(playDurationFormatted);
            $("#songprogress #progress").width(playPercent + "%");
        }, 1000);

        playDurationDelta = 0;
    }

    OnPlayerWaiting = function()
    {
        console.log("WAITING");

        if (stalledStatonCancelTimer != -1)
        {
            clearTimeout(stalledStatonCancelTimer);
        }

        window["CURRENT_PLAYER_STATE"] = PlayerStates.LOADING;
        DispatchPlayerStateUpdatedEvent();
    }

    OnPlayerEnded = function()
    {
        console.log("STOPPED:ENDED");
        window["CURRENT_PLAYER_STATE"] = PlayerStates.STOPPED;
        DispatchPlayerStateUpdatedEvent();
    }

    Init = function(stationId) {

        RadioTime.event.subscribe("schedule", UpdateNowPlayingInfo);

        player = $("#jquery_jplayer_1").jPlayer({
            ready: function(event) {
                ready = true;
            },
            pause: function(event) {
                OnPlayerPause();
            },
            play: function(event)
            {
                OnPlayerPlay();
            },
            stalled: function(event)
            {
                console.log("STALLED");
                // STALLED.. clearing media.. prevent freezing

                console.log(event.jPlayer.status);

                if (event.jPlayer.status.seekPercent == 0
                        && event.jPlayer.status.duration == 0
                        && event.jPlayer.status.currentTime == 0)
                {

                    if (stalledStatonCancelTimer != -1)
                    {
                        clearTimeout(stalledStatonCancelTimer);
                    }

                    stalledStatonCancelTimer = setTimeout(function() {

                        // if(currentStream.indexOf(".mp3") < currentStream.length - 4 &&
                        // 	currentStream.indexOf(".aac") < currentStream.length - 4 &&
                        // 	currentStream.indexOf(".m4a") < currentStream.length - 4)
                        // {
                        $("#jquery_jplayer_1").jPlayer("clearMedia");
                        window["CURRENT_PLAYER_STATE"] = PlayerStates.ERROR;
                        PlayerModule.DispatchPlayerStateUpdatedEvent();
                        // }
                    }, 3750);
                }
            },
            loadedmetadata: function(event)
            {
                window["CURRENT_PLAYER_STATE"] = PlayerStates.PLAYING;
                DispatchPlayerStateUpdatedEvent();

                if (stalledStatonCancelTimer != -1)
                {
                    clearTimeout(stalledStatonCancelTimer);
                }


                if (Math.floor(event.jPlayer.status.currentTime) == 0 && timeUpdateInterval == -1) {
                    SetupTimeUpdateInterval();
                }

            },
            waiting: function()
            {
                OnPlayerWaiting();
            },
            ended: function()
            {
                OnPlayerEnded();
            },
            error: function(event) {
                if (ready && event.jPlayer.error.type === $.jPlayer.error.URL_NOT_SET) {
                    // Setup the media stream again and play it.
                    if (currentStream != null) {

                        var stream = {
                            mp3: currentStream
                        };
                        $("#jquery_jplayer_1").jPlayer("setMedia", stream).jPlayer("play");
                    }

                } else {
                    Stop();
                    raiseEvent("playerModuleLoaded", {currentStation: currentStation});
                    document.getElementById('stationname').innerHTML = "Stream Error";
                    console.log("STREAM ERROR!!! : " + event.jPlayer.error.type);
                }

                window["CURRENT_PLAYER_STATE"] = PlayerStates.ERROR;
                DispatchPlayerStateUpdatedEvent();
                console.log("STOPPED:ERROR");

            },
            timeupdate: function(event) {
                //console.log("timeupdate", playDurationDelta);
            },
            progress: function(event) {
                //console.log("progress");
            },
            onSoundComplete: function(event) {
                console.log("end of track");
            },
            swfPath: "js",
            supplied: "mp3",
            preload: "none",
            wmode: "window",
            cssSelectorAncestor: "",
            cssSelector: {
                play: "#audiocontrol",
                pause: "#pause",
                //  stop: "#stop",
                //  mute: "#mute",
                //  unmute: "#unmute",
                currentTime: "#timeelapsed",
                duration: "#duration"
            },
            volume: 0.6
        });
    }

    UpdateNowPlayingInfo = function() {

        stationName = "";
        showName = "";

        if(currentStation != null) {
            Data.DataServiceMain.GetNowPlayingInfo(currentStation, function(responseData){
				
                if(responseData != null) {

                   var dataArray = responseData.body;

                    paramText = [];

                    for (var i = 0; i < dataArray.length; i++) {
                        paramText[i] = dataArray[i].text;

                        if (dataArray[i].seconds_remaining != null && dataArray[i].duration != null) {
                            secondsRemaining = parseInt(dataArray[i].seconds_remaining);
                            duration = parseInt(dataArray[i].duration);
                            playDurationDelta = 0;
                        }

                        if (dataArray[i].key != null && dataArray[i].key == "station")
                        {
                            stationName = dataArray[i].text;
                        }

                        if (dataArray[i].key != null && dataArray[i].key == "show")
                        {
                            showName = dataArray[i].text;
                        }

                        if (dataArray[i].twitter_id != null) {
                            twitterHandle = dataArray[i].twitter_id;
                        }

                        if (dataArray[i].image != null) {
                            stationLogo = dataArray[i].image;
                        }

                        if (dataArray[i].logo != null) {
                            if (Utils.GeneralUtils.IsNullOrEmpty(stationLogo))
                                stationLogo = dataArray[i].logo;
                            miniPlayerLogo = dataArray[i].logo;
                        }
                    }

                    if (stationName == "")
                    {
                        stationName = dataArray[0].text;
                    }

                    //this means we have the current playing track, so lets split it
                    if (paramText.length >= 5) {
                        genre = paramText[2];
                        country = paramText[3];
                    }
                    else {
                        genre = paramText[1];
                        country = paramText[2];
                    }

                    if (Utils.GeneralUtils.IsNullOrEmpty(showName))
                    {
                        showName = dataArray[1].text == null ? dataArray[2].text : dataArray[1].text;
                    }

                    var __stationTitle = stationName;
                    var __description = dataArray[1].text;
                    var __stationImage = stationLogo;

                    var itemData = {Title: __stationTitle, ImageUrl: __stationImage, Description: __description};
                    itemData.attr = {};
                    itemData.attr["onclick"] = "bb.pushScreen('views/player.html', 'playerView',{stationId : '" + currentStation + "' });";
                    RecentsManager.AddToRecents(itemData);
                }

                DispatchNowPlayingUpdatedEvent();
            });

        }
    }

    function UpdateFavoriteState()
    {
        Data.DataServiceMain.GetNowPlayingInfoFull(currentStation, function(responseData) {
            if (responseData != null) {
                isPreset = responseData.body[0].is_preset;
                DispatchNowPlayingUpdatedEvent();
            }
        });
    }

    DispatchPlayerStateUpdatedEvent = function() {
        raiseEvent("playerStateUpdated", {stationName: stationName});
    }

    DispatchNowPlayingUpdatedEvent = function() {

        var nowPlayingString = "";
        nowPlayingString += Utils.GeneralUtils.IsNullOrEmpty(stationName) ? "" : (stationName + "<br>");
        nowPlayingString += Utils.GeneralUtils.IsNullOrEmpty(showName) ? "" : (showName + "<br>");
        nowPlayingString += Utils.GeneralUtils.IsNullOrEmpty(songName) ? "" : (songName + "<br>");
        nowPlayingString += Utils.GeneralUtils.IsNullOrEmpty(albumName) ? "" : (albumName + "<br>");
        nowPlayingString += Utils.GeneralUtils.IsNullOrEmpty(genre) ? "" : (genre + "<br>");
        nowPlayingString += Utils.GeneralUtils.IsNullOrEmpty(country) ? "" : (country + "<br>");

        // Raise the custom evnent
        raiseEvent("nowPlayingInfoUpdated", {stationName: stationName,
            songName: songName,
            showName: showName,
            artistName: artistName,
            albumName: albumName,
            currentBitrate: currentBitrate,
            currentFormat: currentFormat,
            secondsRemaining: secondsRemaining,
            duration: duration,
            genre: genre,
            stationLogo: stationLogo,
            miniPlayerLogo: miniPlayerLogo,
            country: country,
            isPreset: isPreset,
            nowPlayingString: nowPlayingString});

        DispatchPlayerStateUpdatedEvent();
    }

    SetStation = function(stationId) {

        if (stationId != currentStation) {
            if (currentStation != null) {
                this.ResetPlayer();
            }

            var playDurationFormatted = Utils.GeneralUtils.FormatPlayDuration(0);

            $("#progressinfo #timeelapsed").html(playDurationFormatted);
            $("#songprogress #progress").width("0%");

            currentStation = stationId;

            var playListArray;

            Data.DataServiceMain.GetStationStreams(stationId, function(responseData) {
                if (responseData != null) {
                    streamOptions = responseData.body;
                }

                /* STREAM PROPERTIES
                 * 
                 * URL: "http://opml.radiotime.com/Tune.ashx?id=e6509968&sid=s171560&formats=mp3,aac&partnerId=GdTO4j7I&serial=43A8C489"
                 bitrate: "128"
                 element: "outline"
                 guide_id: "e6509968"
                 media_type: "MP3"
                 now_playing_id: "e6509968"
                 reliability: "92"
                 station_id: "171560"
                 text: "Play | 128 kbps MP3 - 92% reliable"
                 title: "Michael Jackson Radio"
                 type: "audio"
                 */

                if (streamOptions.length > 0) {
                    TuneToStream(streamOptions[0].URL, streamOptions[0].bitrate, streamOptions[0].media_type);
                }
            });


            // Data.DataServiceMain.GetStationTuneUrl(stationId, function(responseData) {
// 				
            // if(responseData != null) {
            // GetStreamAndPlay(responseData);
            // }
            // });

            if (nowPlayingInterval != -1) {
                clearInterval(nowPlayingInterval);
                nowPlayingInterval = -1;
            }
            
            //nowPlayingInterval = setInterval(UpdateNowPlayingInfo, 60 * 1000);

            UpdateNowPlayingInfo();

            RadioTime.schedule.init(stationId);
            UpdateFavoriteState();
        }
        else {
            raiseEvent("playerModuleLoaded", {currentStation: currentStation});
        }

        Manager.LocalStorageManager.setItem("CURRENT_PLAYING_STATION_ID", stationId);
    }

    EnsureSingleUrl = function(responseData)
    {
        var regexp = /http/g;
        var match, matches = [];

        while ((match = regexp.exec(responseData)) != null) {
            matches.push(match.index);
        }

        if (matches.length > 1) {
            responseData = responseData.substring(0, matches[1]);
        }

        return responseData;
    }

    GetStreamAndPlay = function(responseData) {

        responseData = EnsureSingleUrl(responseData);

        if (responseData.indexOf(".pls") > 0) {
            ActivityManager.ShowFeedback("Getting stream...");
            Utils.ParserUtil.DownloadPls(responseData, function(playListUrl) {
                streamUrl = playListUrl;
                streamUrl = Utils.ParserUtil.FormatStream(streamUrl);
                SetStream(streamUrl);
            });
        }
        else if (responseData.indexOf(".m3u") > 0) {
            ActivityManager.ShowFeedback("Getting stream...");
            Utils.ParserUtil.DownloadM3U(responseData, function(playListUrl) {
                streamUrl = playListUrl;
                streamUrl = Utils.ParserUtil.FormatStream(streamUrl);
                SetStream(streamUrl);
            });
        } else {
            streamUrl = responseData.trim();
            streamUrl = Utils.ParserUtil.FormatStream(streamUrl);
            SetStream(streamUrl);
        }
    }

    TuneToStream = function(url, bitrate, mediaType) {

        currentBitrate = bitrate + "K";
        currentFormat = mediaType;

        Data.DataServiceMain.GetStationTuneFromUrl(url, function(responseData) {

            if (responseData != null) {
                GetStreamAndPlay(responseData);
            }
            raiseEvent("playerModuleLoaded", {currentStation: currentStation});
        });
    }

    GetStreamOptions = function() {
        if (streamOptions != null) {
            return streamOptions;
        }
        else {
            return [];
        }
    }

    SetStream = function(streamUrl) {
        
        
        var xhr = new XMLHttpRequest();
        xhr.open("GET", streamUrl, true);
        xhr.setRequestHeader("Icy-Metadata", "1");

        // Timeout to abort in 10 seconds
        var xhrTimeout=setTimeout(function() {
           xhr.abort();
           console.log("AJAX request timed out.");
        }, 10000);

        var headers;
        xhr.onreadystatechange=function(){
           if (xhr.readyState > 2) {
              headers = xhr.getAllResponseHeaders();
              clearTimeout(xhrTimeout);
                if (headers.toLowerCase().indexOf("content-type: text/html") >= 0 ||
                    headers.toLowerCase().indexOf("content-type: audio/mpeg") >= 0){
                    var responseText = xhr.responseText;
                    xhr.abort(); 
                    if(responseText.toLowerCase().indexOf("shoutcast") >= 0){
                        var lastChar = streamUrl.charAt(streamUrl.length-1);
                        var appendChar = "/;";
                        if (lastChar=="/"){
                            appendChar = ";";
                        }
                        streamUrl=streamUrl+appendChar;
                    }
                }
             currentStream = streamUrl;
             xhr.abort();
             PlayStream(currentStream);

           }
        }

        // Now that we're ready to handle the response, we can make the request
        xhr.send(null); 
    }

    PlayStream = function(streamUrl) {

        // does not end in .mp3, .aac, or .m4a
        var isProgressive = !(streamUrl.indexOf(".mp3") < streamUrl.length - 4 &&
                streamUrl.indexOf(".aac") < streamUrl.length - 4 &&
                streamUrl.indexOf(".m4a") < streamUrl.length - 4)

        if (isProgressive)
        {
            if (currentStation != null &&
                    currentStream != null
                    )
            {
                $("#jquery_jplayer_1").html("");
                html5Audio = document.createElement("audio");
                html5Audio.src = currentStream;
                html5Audio.setAttribute("autoplay", true);
                $("#jquery_jplayer_1").append(html5Audio);

                html5Audio.addEventListener("loadstart", function() {
                    console.log("AUDIO ELEM LOAD START");
                    OnPlayerWaiting();
                });

                html5Audio.addEventListener("error", function() {
                    console.log("AUDIO ELEM ERROR!!!")
                });

                html5Audio.addEventListener("waiting", function() {
                    console.log("AUDIO ELEM WAITING");
                    OnPlayerWaiting();
                });

                html5Audio.addEventListener("ended", function() {
                    console.log("AUDIO ELEM ENDED");
                    OnPlayerEnded();
                });

                html5Audio.addEventListener("play", function() {
                    console.log("AUDIO ELEM PLAYED");
                    OnPlayerPlay();

                    window["CURRENT_PLAYER_STATE"] = PlayerStates.PLAYING;
                    DispatchPlayerStateUpdatedEvent();
                });

                html5Audio.addEventListener("pause", function() {
                    console.log("AUDIO ELEM PAUSED");
                    OnPlayerPause();
                });

                html5Audio.addEventListener("loadedmetadata", function(event) {
                    console.log("AUDIO ELEM LOADED METADATA", event);
                    window["CURRENT_PLAYER_STATE"] = PlayerStates.PLAYING;
                    DispatchPlayerStateUpdatedEvent();
                });

                html5Audio.addEventListener("timeupdate", function(event) {
                    //console.log("AUDIO ELEM TIME UPDATE", event)

                    playDuration = event.target.currentTime;
                    duration = event.target.duration;
                    playPercent = Math.round(playDuration * 100 / duration);

                    var playDurationFormatted = Utils.GeneralUtils.FormatPlayDuration(playDuration);

                    $("#progressinfo #timeelapsed").html(playDurationFormatted);
                    $("#songprogress #progress").width(playPercent + "%");

                });

                html5Audio.addEventListener("volumechage", function(event) {
                    console.log("AUDIO ELEM VOLUME CHANGE", event)
                });

            }
        }
        else
        {
            if (currentStation != null &&
                    currentStream != null
                    )
            {
                //clear the feedback fields	
                ActivityManager.ShowFeedback("");

                console.log("stream url::", streamUrl);
                var stream = {
                    mp3: streamUrl
                };

                $("#jquery_jplayer_1").jPlayer("setMedia", stream).jPlayer("play");
            }
        }
    }

    Play = function() {
        if (html5Audio != null)
        {
            html5Audio.play();
        }
        else
        {
            PlayStream(currentStream);
        }

    }

    Stop = function() {


        if (html5Audio != null)
        {
            html5Audio.pause();
        }
        else
        {
            $(player).jPlayer("pause", 0);

            if (timeUpdateInterval != -1) {
                clearInterval(timeUpdateInterval);
                timeUpdateInterval = -1;
            }
        }
    }

    ResetPlayer = function() {
        currentStation = null; // station metadata
        streamOptions = null;
        currentStream = null;
        currentBitrate = "";
        currentFormat = "";
        stationName = "";
        showName = "";
        secondsRemaining = 0;
        playDurationDelta = 0;
        duration = 0;
        playDuration = 0;
        twitterHandle = "";
        stationLogo = "";
        miniPlayerLogo = "";
        window["CURRENT_PLAYER_STATE"] = "";
        isPreset = false;

        if (html5Audio != null)
        {
            html5Audio.pause();
            html5Audio.src = "";
            html5Audio.removeAttribute("autoplay");
            html5Audio = null;
        }

        if (timeUpdateInterval != -1) {
            clearInterval(timeUpdateInterval);
            timeUpdateInterval = -1;
        }

        if (stalledStatonCancelTimer != -1) {
            clearTimeout(stalledStatonCancelTimer);
            stalledStatonCancelTimer = -1;
        }


        Stop();
    }

    GetMiniPlayerDiv = function() {

        /*
         <div>
         <div id="stationimage">
         <img src="images/demos/station-logo1.png">
         </div>
         <div id="nowplayinginfo">
         <span id="stationname">Kiss 92.5</span>
         <span id="songname">Ross and Mocha Morning Show</span>
         </div>
         <div id="audiocontrol" class="playing">
         </div>
         </div>
         */

        var noStationSelected = false;

        var result = document.createElement("div");
        result.setAttribute("style", "height:100%;");

        var stationImageDiv = document.createElement("div");
        stationImageDiv.setAttribute("id", "stationimage");

        var stationImage = document.createElement("img");
        if (miniPlayerLogo != "")
            stationImage.setAttribute("src", miniPlayerLogo);
        else
            stationImage.setAttribute("src", stationLogo);

        stationImageDiv.appendChild(stationImage);

        var nowPlayingInfoDiv = document.createElement("div");
        nowPlayingInfoDiv.setAttribute("id", "nowplayinginfo");

        var stationNameSpan = document.createElement("span");
        stationNameSpan.setAttribute("id", "stationname");

        var songNameSpan = document.createElement("span");
        songNameSpan.setAttribute("id", "songname");


        if (Utils.GeneralUtils.IsNullOrEmpty(currentStation))
        {
            noStationSelected = true;
        }

        if (window["CURRENT_PLAYER_STATE"] == "LOADING")
        {
            stationNameSpan.innerHTML = "Buffering...";
            songNameSpan.innerHTML = "";
        }
        else if (window["CURRENT_PLAYER_STATE"] == "ERROR")
        {
            stationNameSpan.innerHTML = "Stream Error";
            songNameSpan.innerHTML = "";
        }
        else
        {
            if (noStationSelected)
            {
                stationNameSpan.innerHTML = "No station selected";
            }
            else
            {
                stationNameSpan.innerHTML = stationName;

                songNameSpan.innerHTML = showName == "" ? songName : showName;

                if (songNameSpan.innerHTML.length >= 40)
                {
                    songNameSpan.innerHTML = songNameSpan.innerHTML.substring(0, 37) + "..."
                }
            }
        }

        if (!noStationSelected && NavigationManager.GetCurrentScreen().attr("id") != "screen-player")
        {
            result.setAttribute("onclick", "bb.pushScreen('views/player.html', 'playerView', {stationId : '" + PlayerModule.GetCurrentStation() + "'});");
        }
        else
        {
            result.setAttribute("onclick", "Views.PlayerPage.GotoNowPlayingSlide();");
        }



        nowPlayingInfoDiv.appendChild(stationNameSpan);
        nowPlayingInfoDiv.appendChild(songNameSpan);

        var audioControlDiv = document.createElement("div");
        audioControlDiv.setAttribute("id", "audiocontrol");

        if (window["CURRENT_PLAYER_STATE"] == "PLAYING")
        {
            audioControlDiv.setAttribute("class", "stopped"); // show stop action button

            if (!noStationSelected)
            {
                $(audioControlDiv).click(function(e) {
                    PlayerModule.Stop();
                    e.stopPropagation();
                });
            }
        }
        else
        {
            audioControlDiv.setAttribute("class", "playing"); // show play action button

            if (!noStationSelected)
            {
                $(audioControlDiv).click(function(e) {
                    PlayerModule.Play();
                    e.stopPropagation();
                });
            }
        }


        result.appendChild(stationImageDiv);
        result.appendChild(nowPlayingInfoDiv);

        if (!noStationSelected)
            result.appendChild(audioControlDiv);

        return result;
    }

    return {
        Init: Init,
        addListener: addListener,
        UpdateNowPlayingInfo: UpdateNowPlayingInfo,
        SetPlayerState: function(value) {
            window["CURRENT_PLAYER_STATE"] = value;
        },
        GetPlayerState: function() {
            return window["CURRENT_PLAYER_STATE"];
        },
        GetCurrentStation: function() {
            return currentStation;
        },
        GetCurrentTwitterHandle: function() {
            return twitterHandle;
        },
        GetPlayer: function() {
            return player;
        },
        PlayerStates: PlayerStates,
        TuneToStream: TuneToStream,
        Play: Play,
        Stop: Stop,
        SetStation: SetStation,
        GetStreamOptions: GetStreamOptions,
        SetStream: SetStream,
        ResetPlayer: ResetPlayer,
        GetMiniPlayerDiv: GetMiniPlayerDiv,
        IsPreset: function() {
            return isPreset
        },
        UpdateFavoriteState: UpdateFavoriteState,
        DispatchNowPlayingUpdatedEvent: DispatchNowPlayingUpdatedEvent,
        DispatchPlayerStateUpdatedEvent: DispatchPlayerStateUpdatedEvent
    }

}())
