/*
 * This script is for the now playing screen
 * It connects to the Now Playing API with a station ID to get what is now playing
 */

Namespace("Views");
	
Views.PlayerPage = (function(){
	var PageRoot;
	var PlayerSwipe;
	
	function Init(stationId,playing_image) {
            PageRoot = document.getElementById("screen-player"); // Root element of this screen

            PlayerSwipe = new Swipe(document.getElementById('pagecontainer'), {
                startSlide: 0,
                speed: 100,
                callback: function(event, index, elem) {
                    console.log("swipe: ", event, index, elem);
                    Views.PlayerPage.Refresh();
                }
            });
            
            var screenStack = bb.screens;
            for(var i = 0; i < screenStack.length - 1; i++){  //check all screens except for the current one,which is playerView
                if (screenStack[i].id == "playerView" || screenStack[i].id == "relatedSubcategory" ||
                            screenStack[i].id == "chooseStream" || screenStack[i].id == "nowPlaying" ||
                            screenStack[i].id == "favoritesSubcategory" || screenStack[i].id == "searchView"){
                       bb.screens.removeAt(i);
                }
            }
            // Disable add to favorites for podcasts
            if(!Utils.GeneralUtils.IsNullOrEmpty(stationId))
            {
                    if (stationId[0] == "t")
                    {
                            document.getElementById("favoritesTag").onclick = "";
                            $("#favoritesTag").css("opacity", "0.35");
                    }
            }
        
        PlayerModule.addListener("nowPlayingInfoUpdated", UpdateNowPlaying);

        if(PlayerModule.GetCurrentStation() != stationId)
        {
        	PlayerModule.SetStation(stationId);
        }
		else
		{
			PlayerModule.DispatchNowPlayingUpdatedEvent();
			$("#loading-overlay").addClass("hidden");
		}

		
		PlayerModule.addListener("playerModuleLoaded", function(){
			$("#loading-overlay").addClass("hidden");
		});

		UpdateMiniPlayerContent();
        PlayerModule.addListener("playerStateUpdated", UpdateMiniPlayerContent);
		
		//UpdateNowPlaying(stationId, playing_image);

		
		Views.PlayerPage.UpdateFavoriteState();

		Views.PlayerPage.Refresh();
		
	}

	function UpdateMiniPlayerContent()
    {
        $(PageRoot).find("#tunein-playerbar").html("");
        $(PageRoot).find("#tunein-playerbar").append(PlayerModule.GetMiniPlayerDiv());
    }
	
    function UpdateNowPlaying(eventName, args){
			
		console.log("PLAYER NOW PLAYING:", args);
		//$("#now-playing-details-panel").html(nowPlayingString);
		$("#trackinfo .albumname").html(args.stationName);
		$("#trackinfo #songname").html(Utils.GeneralUtils.IsNullOrEmpty(args.showName) ? "" : args.showName);
		$("#additionalinfo #artistname").html(Utils.GeneralUtils.IsNullOrEmpty(args.songName) ? (Utils.GeneralUtils.IsNullOrEmpty(args.showName) ? "" : args.showName) : args.songName);
		$("#streaminfo #bitrate").html(args.currentBitrate);
		$("#streaminfo #format").html(args.currentFormat);
		$("#albumimage img").attr("src", args.stationLogo);
		$("#albumimage").css("background-image", "url('" + args.stationLogo + "')");

		UpdateFavoriteState();
	}

	function Refresh() {
		if(PlayerSwipe != null) {
			var elem = PlayerSwipe.slides[PlayerSwipe.index];
			if(elem != null)
			{
				if(elem.id == "player") {
					$("#screen-player .refresh").hide();
				}
			else if(elem.id == "screen-player-twitterfeed") {
	                Views.TwitterFeed.Init();
	                $("#screen-player .refresh").show();
	            } else if(elem.id == "screen-player-related") {
	                Views.RelatedPage.Init();
	                $("#screen-player .refresh").show()
	            } else if(elem.id == "screen-player-favorites") {
	                Views.FavoritesPage.Init();
	                $("#screen-player .refresh").show();
	            } else if(elem.id == "screen-player-recents"){
	                Views.RecentsPage.Init();
	                $("#screen-player .refresh").show();
	            }
	            else
	            {
	            	$("#screen-player .refresh").hide();
	            }
			}
		}
		else{
			// do nothing?
		}
	}

	UpdateFavoriteState = function() {

		$(document.getElementById('favoritesTag')).removeClass("add_favorite");
		$(document.getElementById('favoritesTag')).removeClass("remove_favorite");

		if (FavoritesManager.FavoriteExists()){
			var favTag = document.getElementById('favoritesTag');
			if(favTag != null)
			{
				favTag.setImage("images/icons/removefavorites.png");
				favTag.setCaption("Remove station from favorites");
				$(favTag).addClass("remove_favorite");
			}
		}
		else {
			var favTag = document.getElementById('favoritesTag');
			if(favTag != null)
			{
				favTag.setImage("images/icons/addtofavorites.png");
				$(favTag).addClass("add_favorite");
				favTag.setCaption("Add station to favorites");
			}
		}
	}
	
	showPlayerOptions = function() {
		bb.pushScreen('views/player-options.html', 'playerOptions');
	}

	GotoNowPlayingSlide = function()
	{
		// for (var i = 0; i < PlayerSwipe.index; i++)
		// {
		// 	PlayerSwipe.prev();
		// }

		PlayerSwipe = new Swipe(document.getElementById('pagecontainer'), {
            startSlide: 0,
            speed: 100,
            callback: function(event, index, elem) {
            	
            	console.log("swipe: ", event, index, elem);
                
                Views.PlayerPage.Refresh();
            }
        });
	}
	
	return {
		Init: Init,
		showPlayerOptions : showPlayerOptions,
		Refresh : Refresh,
		UpdateFavoriteState : UpdateFavoriteState,
		GetPlayerSwipe : function(){return PlayerSwipe;},
		GotoNowPlayingSlide : GotoNowPlayingSlide
	}
	
})();

Views.NowPlaying = (function(){

	var PageRoot;

	Init = function() {
		PageRoot = document.getElementById("screen-now-playing"); // Root element of this screen
		
		PlayerModule.addListener("nowPlayingInfoUpdated", function(eventName, args){
			
			// var nowPlayingString = "";
				// nowPlayingString += args.stationName == null ? "" : args.stationName + "<br>";
				// nowPlayingString += args.showName == null ? "" : args.showName + "<br>";
				// nowPlayingString += args.songName == null ? "" : args.songName + "<br>";
				// nowPlayingString += args.albumName == null ? "" : args.albumName + "<br>";
				// nowPlayingString += args.genre == null ? "" : args.genre + "<br>";
				// nowPlayingString += args.country == null ? "" : args.country + "<br>";
				
			var nowPlayingString = args.nowPlayingString;
				
			$("#now-playing-details-panel").html(nowPlayingString);
			
			$("#streaminfo #bitrate").html(args.currentBitrate);
			$("#streaminfo #format").html(args.currentFormat);
		});
		
		PlayerModule.DispatchNowPlayingUpdatedEvent();

		UpdateMiniPlayerContent();
        PlayerModule.addListener("playerStateUpdated", UpdateMiniPlayerContent);
	}

	function UpdateMiniPlayerContent()
    {
        $(PageRoot).find("#tunein-playerbar").html("");
        $(PageRoot).find("#tunein-playerbar").append(PlayerModule.GetMiniPlayerDiv());
    }
	
	return {
		Init: Init
	}
	
}());

Views.ChooseStream = (function(){
	
	var PageRoot;

	Init = function() {
		PageRoot = document.getElementById("screen-choose-stream"); // Root element of this screen
		
		if(PlayerModule.GetCurrentStation() != null) {
			var streamOptions = PlayerModule.GetStreamOptions();
			
			streamOptionsDataSource = [];
			
			for (var i = 0; i < streamOptions.length; i++) {
				streamOptionsDataSource.push({Title: streamOptions[i].text, image: "", subtext: "", attr:{"data-tunein-type": "no_image_item", "data-custom-url": streamOptions[i].URL, "data-custom-bitrate":streamOptions[i].bitrate, "data-custom-mediaType":streamOptions[i].media_type}});
			}
			
			streamsList = BBUI_Helper.CreateList(streamOptionsDataSource);
            document.getElementById("StreamsListContainer").appendChild(streamsList);
            
            $(document.getElementById("StreamsListContainer")).find("div[data-bb-type='item']").each(function(index){
            	this.addEventListener("click", Views.ChooseStream.OnStreamClick, true);  
            });
            
            bb.doLoad(document.getElementById("StreamsListContainer"));
		}

		UpdateMiniPlayerContent();
        PlayerModule.addListener("playerStateUpdated", UpdateMiniPlayerContent);
	}

	function UpdateMiniPlayerContent()
    {
        $(PageRoot).find("#tunein-playerbar").html("");
        $(PageRoot).find("#tunein-playerbar").append(PlayerModule.GetMiniPlayerDiv());
    }
	
	function OnStreamClick(event) {
		PlayerModule.TuneToStream(event.currentTarget.getAttribute("data-custom-url"), event.currentTarget.getAttribute("data-custom-bitrate"), event.currentTarget.getAttribute("data-custom-mediaType"));
	}
	
	return {
		Init: Init,
		OnStreamClick : OnStreamClick
	}
	
}());


//sample response///
/*
<?xml version="1.0" encoding="UTF-8"?>
<opml version="1">
	<head>
	<status>200</status>
	
	</head>
	<body>
<outline type="text" text="Pop Fusion" guide_id="s112498" key="station" image="http://d1i6vahw24eb07.cloudfront.net/s112498q.png" preset_id="s112498"/>
<outline type="text" text="One Direction - What Makes You Beautiful"/>
<outline type="text" text="Where the hits happen"/>
<outline type="text" text="Chicago, IL"/>
<outline type="object" text="NowPlaying">
<nowplaying>
  <logo>http://d1i6vahw24eb07.cloudfront.net/s112498.png</logo>
  <twitter_id>pop_fusion_</twitter_id>
</nowplaying>
</outline>
	</body>
</opml>
*/