/*
 * This script is for the search screen
 */

Namespace("Views");

Views.SearchPage = (function() {
    var PageRoot = null;
    const ENTER = 13;

    function Init(searchParams) {
        PageRoot = document.getElementById("screen-search"); // Root element of this screen
        var searchTerm = "";
        var url = "";
        //called when the users clicks the 'back' button
        if (searchParams != null && searchParams.searchTerm != null && searchParams.searchTerm != '')
        {
            searchTerm = searchParams.searchTerm;
        }
        else if (searchParams != null && searchParams.url != null && searchParams.url != '')
        {
            url = searchParams.url;
        }
        else
        {
            searchTerm = Manager.LocalStorageManager.getItem('CURRRENT_SEARCH_TERM');
        }
        
        //document.addEventListener('keypress', enterKeyPressed, false);
        
        $("#search-query").unbind('keypress', enterKeyPressed);
        $("#search-query").bind('keypress', enterKeyPressed);
        
        UpdateMiniPlayerContent();
        PlayerModule.addListener("playerStateUpdated", UpdateMiniPlayerContent);

        Views.SearchPage.doSearch(searchTerm, url);
    }

    function UpdateMiniPlayerContent()
    {
        $(PageRoot).find("#tunein-playerbar").html("");
        $(PageRoot).find("#tunein-playerbar").append(PlayerModule.GetMiniPlayerDiv());
    }

    function doSearch(searchTerm, url) {

        //clear any feedback we might have
        ActivityManager.ShowFeedback("");

        //save our searchTerm so we can reuse it on the back button	
        if (searchTerm != "")
        {
            Manager.LocalStorageManager.setItem('CURRRENT_SEARCH_TERM', searchTerm);
            document.getElementById('search-query').value = searchTerm;
        }

        var requestUrl = Utils.Constants.TUNE_IN_API + "Search.ashx";

        var params = "partnerId=" + Utils.Constants.PARTNER_ID + "&query=" + searchTerm + "&formats=mp3,aac";

        //if there is a url then we are reloading the serach page, but for an artist//
        if (url != "")
        {
            var urlParams = url.split("?");
            params = urlParams[1];
            searchTerm = "-";//put something for the term so the search happens again
            requestUrl = urlParams[0];
        }

        if (searchTerm == "")
        {
            //Manager.LocalStorageManager.removeItem('CURRRENT_SEARCH_TERM');
        }
        else
        {
            requestUrl = requestUrl + "?" + params;

            Data.DataServiceMain.DoRequestToUrl(requestUrl, function(responseData)
            {
                var url, text, audioType, guide_id, playing_image, bitrate = "";
                //if the status is 200 then all is good, lets proceed

                ActivityManager.HideIndicator();
                ActivityManager.ClearFeedback();

                if (responseData != null)
                {
                    //clear our list of any old results
                    document.getElementById('dataList').innerHTML = "";

                    // Create our list item
                    for (var idx in responseData.body) {
                        var category = responseData.body[idx];

                        //show the category labels
                        var dataList = document.getElementById('dataList');

                        var listItem = CreateListItem(category);

                        dataList.appendItem(listItem);

                        if (category.children != null) {
                            for (var i = 0; i < category.children.length; i++)
                            {
                                var item = category.children[i];
                                var listItem = CreateListItem(item);
                                dataList.appendItem(listItem);
                            }
                        }
                    }
                    ActivityManager.ShowFeedback("");
                }
                else
                {
                    ActivityManager.HideIndicator();
                    ActivityManager.ShowFeedback("Search unavailable. Please try again later.");
                }
            });
            ActivityManager.ShowFeedback("Searching...");
        }
    }
    
    function enterKeyPressed(e) {
    	if (e.keyCode == ENTER || e.which == ENTER)
    	{
	        var searchparams = {
	        	"searchTerm" : document.getElementById('search-query').value
	        };
	        
                var activeElement = document.activeElement;
                if (activeElement) {
                  if (activeElement.tagName == 'INPUT') {
                    activeElement.blur();
                  }
                }
            
	        if(searchparams.searchTerm != undefined || searchparams.searchTerm != ""){
	            Views.SearchPage.Init(searchparams);
           	}
        } 
            
    } 

    function CreateListItem(item) {
        var listItem = document.createElement('div');

        var itemType = $(item).attr('type');

        if (itemType != null && itemType == "link")
        {
            listItem.setAttribute('data-bb-type', 'item');
            listItem.setAttribute('data-bb-title', $(item).attr('text'));
            listItem.setAttribute('onclick', "bb.pushScreen('views/search.html', 'searchView', {searchTerm:'',url:'" + $(item).attr('URL') + "'});");
            listItem.setAttribute("data-tunein-type", "no_image_item");
        }
        else if (itemType != null && itemType == "audio")
        {
            playing_image = $(item).attr('image');
            bitrate = $(item).attr('bitrate');
            guide_id = $(item).attr('guide_id');

            listItem.setAttribute('data-bb-type', 'item');
            listItem.setAttribute('data-bb-img', $(item).attr('image'));
            listItem.setAttribute('data-bb-title', $(item).attr('text'));
            
            if($(item).attr("text").toLowerCase().indexOf("not supported") == -1)
            {
                listItem.setAttribute('onclick', "bb.pushScreen('views/player.html', 'playerView',{stationId : '" + guide_id + "',playing_image : '" + playing_image + "' });");
            }
        
            listItem.innerHTML = $(item).attr('subtext');

        }
        else {
            if(!Utils.GeneralUtils.IsNullOrEmpty($(item).attr('text')))
            {
                listItem.setAttribute('class', 'sectionheader');
                listItem.innerHTML = $(item).attr('text');    
            }
        }

        return listItem;
    }

    return {
        Init: Init,
        doSearch: doSearch,
        enterKeyPressed: enterKeyPressed
    }
})();


